@@ -14,6 +14,7 @@ from httpx import AsyncClient
 
 from meilisearch_python_async._http_requests import _HttpRequests
 from meilisearch_python_async.errors import (
+    InvalidDocumentError,
     MeiliSearchApiError,
     MeiliSearchError,
     MeiliSearchTimeoutError,
@@ -56,6 +57,10 @@ class Index:
         self.primary_key = primary_key
         self.created_at: datetime | None = Index._iso_to_date_time(created_at)
         self.updated_at: datetime | None = Index._iso_to_date_time(updated_at)
+        self._base_url = "indexes/"
+        self._base_url_with_uid = f"{self._base_url}{self.uid}"
+        self._documents_url = f"{self._base_url_with_uid}/documents"
+        self._stats_url = f"{self._base_url_with_uid}/stats"
         self._http_requests = _HttpRequests(http_client)
 
     def __str__(self) -> str:
@@ -74,7 +79,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}"
+        url = f"{self._base_url_with_uid}"
         response = await self._http_requests.delete(url)
         return response.status_code
 
@@ -113,7 +118,7 @@ class Index:
         if primary_key is not None:
             payload["primaryKey"] = primary_key
 
-        url = f"indexes/{self.uid}"
+        url = f"{self._base_url_with_uid}"
         response = await self._http_requests.put(url, payload)
         self.primary_key = response.json()["primaryKey"]
         return self
@@ -128,7 +133,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}"
+        url = f"{self._base_url_with_uid}"
         response = await self._http_requests.get(url)
         index_dict = response.json()
         self.primary_key = index_dict["primaryKey"]
@@ -199,7 +204,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/updates"
+        url = f"{self._base_url_with_uid}/updates"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -220,7 +225,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/updates/{update_id}"
+        url = f"{self._base_url_with_uid}/updates/{update_id}"
         response = await self._http_requests.get(url)
 
         return UpdateStatus(**response.json())
@@ -267,7 +272,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/stats"
+        url = f"{self._stats_url}"
         response = await self._http_requests.get(url)
 
         return IndexStats(**response.json())
@@ -322,7 +327,7 @@ class Index:
             "attributesToHighlight": attributes_to_highlight,
             "matches": matches,
         }
-        url = f"indexes/{self.uid}/search"
+        url = f"{self._base_url_with_uid}/search"
         response = await self._http_requests.post(url, body=body)
 
         return SearchResults(**response.json())
@@ -340,7 +345,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents/{document_id}"
+        url = f"{self._documents_url}/{document_id}"
         response = await self._http_requests.get(url)
         return response.json()
 
@@ -370,7 +375,7 @@ class Index:
         if attributes_to_retrieve:
             parameters["attributesToRetrieve"] = attributes_to_retrieve
 
-        url = f"indexes/{self.uid}/documents?{urlencode(parameters)}"
+        url = f"{self._documents_url}?{urlencode(parameters)}"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -395,7 +400,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents"
+        url = f"{self._documents_url}"
         if primary_key:
             formatted_primary_key = urlencode({"primaryKey": primary_key})
             url = f"{url}?{formatted_primary_key}"
@@ -462,6 +467,173 @@ class Index:
 
         return update_ids
 
+    async def add_documents_from_directory(
+        self,
+        directory_path: Path | str,
+        *,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and add the documents to the index.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            Update id to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            response = await self.add_documents(combined, primary_key)
+            return [response]
+
+        responses = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.add_documents(documents, primary_key)
+                responses.append(response)
+
+        return responses
+
+    async def add_documents_from_directory_auto_batch(
+        self,
+        directory_path: Path | str,
+        *,
+        max_payload_size: int = 104857600,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and add the documents to the index.
+
+        Documents are automatically split into batches as large as possible based on the max payload
+        size.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            max_payload_size: The maximum payload size in bytes. Defaults to 104857600.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            List of update ids to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            return await self.add_documents_auto_batch(
+                combined, max_payload_size=max_payload_size, primary_key=primary_key
+            )
+
+        responses: list[UpdateId] = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.add_documents_auto_batch(
+                    documents, max_payload_size=max_payload_size, primary_key=primary_key
+                )
+                responses = [*responses, *response]
+
+        return responses
+
+    async def add_documents_from_directory_in_batches(
+        self,
+        directory_path: Path | str,
+        *,
+        batch_size: int = 1000,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and add the documents to the index in batches.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            batch_size: The number of documents that should be included in each batch.
+                Defaults to 1000.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            List of update ids to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            return await self.add_documents_in_batches(
+                combined, batch_size=batch_size, primary_key=primary_key
+            )
+
+        responses: list[UpdateId] = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.add_documents_in_batches(
+                    documents, batch_size=batch_size, primary_key=primary_key
+                )
+                responses = [*responses, *response]
+
+        return responses
+
     async def add_documents_from_file(
         self, file_path: Path | str, primary_key: str | None = None
     ) -> UpdateId:
@@ -469,13 +641,15 @@ class Index:
 
         Args:
             file_path: Path to the json file.
-            primary_key The primary key of the documents. This will be ignored if already set.
+            primary_key: The primary key of the documents. This will be ignored if already set.
                 Defaults to None.
 
         Returns:
             Update id to track the action.
 
         Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
@@ -504,6 +678,7 @@ class Index:
             List of update ids to track the action.
 
         Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
             MeiliSearchError: If the file path is not valid.
             MeiliSearchCommunicationError: If there was an error communicating with the server.
             MeiliSearchApiError: If the MeiliSearch API returned an error.
@@ -534,6 +709,7 @@ class Index:
             List of update ids to track the action.
 
         Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
             MeiliSearchError: If the file path is not valid
             MeiliSearchCommunicationError: If there was an error communicating with the server.
             MeiliSearchApiError: If the MeiliSearch API returned an error.
@@ -561,7 +737,7 @@ class Index:
             MeiliSearchCommunicationError: If there was an error communicating with the server.
             MeiliSearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents"
+        url = f"{self._documents_url}"
         if primary_key:
             formatted_primary_key = urlencode({"primaryKey": primary_key})
             url = f"{url}?{formatted_primary_key}"
@@ -630,6 +806,173 @@ class Index:
 
         return update_ids
 
+    async def update_documents_from_directory(
+        self,
+        directory_path: Path | str,
+        *,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and update the documents.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            Update id to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            response = await self.update_documents(combined, primary_key)
+            return [response]
+
+        responses = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.update_documents(documents, primary_key)
+                responses.append(response)
+
+        return responses
+
+    async def update_documents_from_directory_auto_batch(
+        self,
+        directory_path: Path | str,
+        *,
+        max_payload_size: int = 104857600,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and update the documents.
+
+        Documents are automatically split into batches as large as possible based on the max payload
+        size.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            max_payload_size: The maximum payload size in bytes. Defaults to 104857600.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            List of update ids to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            return await self.update_documents_auto_batch(
+                combined, max_payload_size=max_payload_size, primary_key=primary_key
+            )
+
+        responses: list[UpdateId] = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.update_documents_auto_batch(
+                    documents, max_payload_size=max_payload_size, primary_key=primary_key
+                )
+                responses = [*responses, *response]
+
+        return responses
+
+    async def update_documents_from_directory_in_batches(
+        self,
+        directory_path: Path | str,
+        *,
+        batch_size: int = 1000,
+        primary_key: str | None = None,
+        combine_documents: bool = True,
+    ) -> list[UpdateId]:
+        """Load all json files from a directory and update the documents.
+
+        Args:
+            directory_path: Path to the directory that contains the json files.
+            batch_size: The number of documents that should be included in each batch.
+                Defaults to 1000.
+            primary_key: The primary key of the documents. This will be ignored if already set.
+                Defaults to None.
+            combine_documents: If set to True this will combine the documents from all the files
+                before indexing them. Defaults to True.
+
+        Returns:
+            List of update ids to track the action.
+
+        Raises:
+            InvalidDocumentError: If the docucment is not a valid format for MeiliSarch.
+            MeiliSearchError: If the file path is not valid
+            MeilisearchCommunicationError: If there was an error communicating with the server.
+            MeilisearchApiError: If the MeiliSearch API returned an error.
+        """
+        directory = Path(directory_path) if isinstance(directory_path, str) else directory_path
+
+        if combine_documents:
+            all_documents = []
+            for path in directory.iterdir():
+                if path.suffix == ".json":
+                    documents = await Index._load_documents_from_file(path)
+                    all_documents.append(documents)
+            loop = get_running_loop()
+            combined = await loop.run_in_executor(
+                None, partial(Index._combine_documents, all_documents)
+            )
+
+            return await self.update_documents_in_batches(
+                combined, batch_size=batch_size, primary_key=primary_key
+            )
+
+        responses: list[UpdateId] = []
+
+        for path in directory.iterdir():
+            if path.suffix == ".json":
+                documents = await Index._load_documents_from_file(path)
+                response = await self.update_documents_in_batches(
+                    documents, batch_size=batch_size, primary_key=primary_key
+                )
+                responses = [*responses, *response]
+
+        return responses
+
     async def update_documents_from_file(
         self, file_path: Path | str, primary_key: str | None = None
     ) -> UpdateId:
@@ -723,7 +1066,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents/{document_id}"
+        url = f"{self._documents_url}/{document_id}"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -741,7 +1084,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents/delete-batch"
+        url = f"{self._documents_url}/delete-batch"
         response = await self._http_requests.post(url, ids)
 
         return UpdateId(**response.json())
@@ -756,7 +1099,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/documents"
+        url = f"{self._documents_url}"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -771,7 +1114,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings"
+        url = f"{self._base_url_with_uid}/settings"
         response = await self._http_requests.get(url)
 
         return MeiliSearchSettings(**response.json())
@@ -791,7 +1134,7 @@ class Index:
         """
         body_dict = {k: v for k, v in body.dict(by_alias=True).items() if v is not None}
 
-        url = f"indexes/{self.uid}/settings"
+        url = f"{self._base_url_with_uid}/settings"
         response = await self._http_requests.post(url, body_dict)
 
         return UpdateId(**response.json())
@@ -806,7 +1149,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings"
+        url = f"{self._base_url_with_uid}/settings"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -821,7 +1164,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/ranking-rules"
+        url = f"{self._base_url_with_uid}/settings/ranking-rules"
         response = await self._http_requests.get(url)
 
         return response.json()
@@ -839,7 +1182,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/ranking-rules"
+        url = f"{self._base_url_with_uid}/settings/ranking-rules"
         respose = await self._http_requests.post(url, ranking_rules)
 
         return UpdateId(**respose.json())
@@ -854,7 +1197,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/ranking-rules"
+        url = f"{self._base_url_with_uid}/settings/ranking-rules"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -870,7 +1213,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/distinct-attribute"
+        url = f"{self._base_url_with_uid}/settings/distinct-attribute"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -891,7 +1234,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/distinct-attribute"
+        url = f"{self._base_url_with_uid}/settings/distinct-attribute"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -906,7 +1249,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/distinct-attribute"
+        url = f"{self._base_url_with_uid}/settings/distinct-attribute"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -921,7 +1264,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/searchable-attributes"
+        url = f"{self._base_url_with_uid}/settings/searchable-attributes"
         response = await self._http_requests.get(url)
         return response.json()
 
@@ -938,7 +1281,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/searchable-attributes"
+        url = f"{self._base_url_with_uid}/settings/searchable-attributes"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -956,7 +1299,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/searchable-attributes"
+        url = f"{self._base_url_with_uid}/settings/searchable-attributes"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -971,7 +1314,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/displayed-attributes"
+        url = f"{self._base_url_with_uid}/settings/displayed-attributes"
         response = await self._http_requests.get(url)
         return response.json()
 
@@ -988,7 +1331,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/displayed-attributes"
+        url = f"{self._base_url_with_uid}/settings/displayed-attributes"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -1003,7 +1346,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/displayed-attributes"
+        url = f"{self._base_url_with_uid}/settings/displayed-attributes"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -1018,7 +1361,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/stop-words"
+        url = f"{self._base_url_with_uid}/settings/stop-words"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -1039,7 +1382,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/stop-words"
+        url = f"{self._base_url_with_uid}/settings/stop-words"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -1054,7 +1397,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/stop-words"
+        url = f"{self._base_url_with_uid}/settings/stop-words"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -1069,7 +1412,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/synonyms"
+        url = f"{self._base_url_with_uid}/settings/synonyms"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -1090,7 +1433,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/synonyms"
+        url = f"{self._base_url_with_uid}/settings/synonyms"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -1105,7 +1448,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/synonyms"
+        url = f"{self._base_url_with_uid}/settings/synonyms"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -1120,7 +1463,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/filterable-attributes"
+        url = f"{self._base_url_with_uid}/settings/filterable-attributes"
         response = await self._http_requests.get(url)
 
         if not response.json():
@@ -1141,7 +1484,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/filterable-attributes"
+        url = f"{self._base_url_with_uid}/settings/filterable-attributes"
         response = await self._http_requests.post(url, body)
 
         return UpdateId(**response.json())
@@ -1156,7 +1499,7 @@ class Index:
             MeilisearchCommunicationError: If there was an error communicating with the server.
             MeilisearchApiError: If the MeiliSearch API returned an error.
         """
-        url = f"indexes/{self.uid}/settings/filterable-attributes"
+        url = f"{self._base_url_with_uid}/settings/filterable-attributes"
         response = await self._http_requests.delete(url)
 
         return UpdateId(**response.json())
@@ -1167,6 +1510,10 @@ class Index:
         for i in range(0, total_len, batch_size):
             yield documents[i : i + batch_size]
 
+    @staticmethod
+    def _combine_documents(documents: list[list[Any]]) -> list[Any]:
+        return [x for y in documents for x in y]
+
     @staticmethod
     async def _generate_auto_batches(
         documents: list[dict[str, Any]], max_payload_size: int
@@ -1221,7 +1568,9 @@ class Index:
             return datetime.strptime(reduced, "%Y-%m-%dT%H:%M:%S.%fZ")
 
     @staticmethod
-    async def _load_documents_from_file(file_path: Path | str) -> list[dict]:
+    async def _load_documents_from_file(
+        file_path: Path | str,
+    ) -> list[dict[str, Any]]:
         if isinstance(file_path, str):
             file_path = Path(file_path)
 
@@ -1230,7 +1579,12 @@ class Index:
 
         async with aiofiles.open(file_path, mode="r") as f:
             data = await f.read()
-            return await loop.run_in_executor(None, partial(json.loads, data))
+            documents = await loop.run_in_executor(None, partial(json.loads, data))
+
+            if not isinstance(documents, list):
+                raise InvalidDocumentError("MeiliSearch requires documents to be in a list")
+
+            return documents
 
     @staticmethod
     def _validate_json_path(file_path: Path) -> None:
